/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.helm;

import java.io.File;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.service.App;
import org.springframework.beans.factory.annotation.Autowired;

// Supports only helm3 + version
public class HelmClient {

    private final ApplicationConfig appConfig;

    public HelmClient(@Autowired ApplicationConfig appConfig) {
        this.appConfig = appConfig;
    }

    public void installApp(App app) throws ServiceException {
        ProcessBuilder builder = prepareInstallCommand(app);
        executeCommand(builder);
    }

    public void uninstallApp(App app) throws ServiceException {
        ProcessBuilder builder = prepareUnInstallCommand(app);
        executeCommand(builder);
    }

    protected String getOverrideFileName(App app) {
        Path appPath = getAppPath(app);
        File file = new File(appPath.toFile(), "values.yaml");
        return file.getAbsolutePath();

    }

    protected String getChartFileName(App app) {
        return getChartFile(app).getAbsolutePath();
    }

    protected String releaseName(App app) {
        return app.getName() + "-" + app.getVersion();
    }

    protected File getChartFile(App app) {
        Path appPath = getAppPath(app);
        return new File(appPath.toFile(), "chart.tgz");
    }

    private Path getAppPath(App app) {
        return Path.of(getDatabaseDirectory(), app.getName(), app.getVersion());
    }

    private String getDatabaseDirectory() {
        return appConfig.getVardataDirectory() + "/database";
    }

    private String executeCommand(ProcessBuilder builder) throws ServiceException {
        try {
            Process process = builder.start();
            process.waitFor();
            int exitValue = process.exitValue();
            String output = IOUtils.toString(process.getInputStream(), StandardCharsets.UTF_8);

            if (exitValue != 0) {
                String error = IOUtils.toString(process.getErrorStream(), StandardCharsets.UTF_8);
                throw new ServiceException("Command execution failed: " + builder + " " + error);
            }
            return output;
        } catch (Exception e) {
            throw new ServiceException("Failed to execute the Command", e);
        }
    }

    public ProcessBuilder prepareInstallCommand(App app) {

        List<String> helmArguments = Arrays.asList("helm", //
            "install", releaseName(app), getChartFileName(app), //
            "--version", app.getVersion(), //
            "--namespace", app.getNamespace());
        addKubeApiInfo(helmArguments);
        ProcessBuilder processBuilder = new ProcessBuilder();
        return processBuilder.command(helmArguments);
    }

    public ProcessBuilder prepareUnInstallCommand(App app) {
        List<String> helmArguments = Arrays.asList("helm", "uninstall", releaseName(app), //
            "--namespace", app.getNamespace());
        addKubeApiInfo(helmArguments);
        ProcessBuilder processBuilder = new ProcessBuilder();
        return processBuilder.command(helmArguments);
    }

    private void addKubeApiInfo(List<String> helmArguments) {
        final String KUBE_API = "--kube-apiserver";
        final String KUBE_TOKEN = "--kube-token";
        final String KUBE_CERT = "--kube-ca-file";

        List<String> kubeApiInfo = Arrays.asList(KUBE_API, "", KUBE_TOKEN, "", KUBE_CERT, "");
        helmArguments.addAll(kubeApiInfo);
    }

}
