/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.service;

import java.io.File;
import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.controller.AppInfo;
import org.oransc.rappmanager.exception.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.multipart.MultipartFile;

@Component
public class AppStore {

    private final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    /**
     * The appStore map contains app name as key & App as value
     */
    private Map<String, App> appMap = new HashMap<>();

    @Autowired
    ApplicationConfig appConfig;

    public synchronized App saveApp(AppInfo appInfo, MultipartFile chartFile, MultipartFile overrideFile)
        throws IOException, ServiceException {
        if (appMap.containsKey(key(appInfo.getName(), appInfo.getVersion()))) {
            throw new ServiceException("App Already Exist");
        }
        Path appPath = getAppPath(appInfo.getName(), appInfo.getVersion());
        Files.createDirectories(appPath);

        chartFile.transferTo(new File(appPath.toFile(), "chart.tgz"));
        overrideFile.transferTo(new File(appPath.toFile(), "values.yaml"));

        App app = App.builder() //
            .name(appInfo.getName()) //
            .version(appInfo.getVersion()) //
            .namespace(appInfo.getNamespace()) //
            .build();

        appMap.put(key(app), app);
        return app;
    }

    public synchronized App getApp(String name, String version) throws ServiceException {
        App app = appMap.get(key(name, version));
        if (app == null) {
            throw new ServiceException("App not found: " + name);
        }
        return app;
    }

    public synchronized List<App> getAllApp() {
        return new ArrayList<>(appMap.values());
    }

    public synchronized void deleteApp(App app) {
        try {
            Path appPath = getAppPath(app.getName(), app.getVersion());
            FileSystemUtils.deleteRecursively(appPath);
        } catch (IOException e) {
            logger.warn("Could not delete app database : {}", e.getMessage());
        }

        appMap.remove(key(app));
    }

    public synchronized void reset() {
        try {
            FileSystemUtils.deleteRecursively(Path.of(this.appConfig.getVardataDirectory()));
        } catch (IOException e) {
            logger.warn("Could not delete database : {}", e.getMessage());
        }
        appMap.clear();
    }

    private Path getAppPath(String appName, String appVersion) {
        return Path.of(getDatabaseDirectory(), appName, appVersion);
    }

    private String getDatabaseDirectory() {
        return appConfig.getVardataDirectory() + "/database";
    }

    private String key(App app) {
        return key(app.getName(), app.getVersion());
    }

    private String key(String appName, String appVersion) {
        return appName + "_" + appVersion;
    }

}
